@@ -18,6 +18,12 @@
 /*	be tried again at a later time. Delivery status reports are sent
 /*	to the \fBbounce\fR(8), \fBdefer\fR(8) or \fBtrace\fR(8) daemon as
 /*	appropriate.
+/* CASE FOLDING
+/* .ad
+/* .fi
+/*	All delivery decisions are made using the bare recipient
+/*	name (i.e. the address localpart), folded to lower case.
+/*	See also under ADDRESS EXTENSION below for a few exceptions.
 /* SYSTEM-WIDE AND USER-LEVEL ALIASING
 /* .ad
 /* .fi
@@ -33,7 +39,7 @@
 /*	configuration parameter. Upon delivery, the local delivery agent
 /*	tries each pathname in the list until a file is found.
 /*
-/*	Delivery via ~/.\fB.forward\fR files is done with the privileges
+/*	Delivery via ~/.\fBforward\fR files is done with the privileges
 /*	of the recipient.
 /*	Thus, ~/.\fBforward\fR like files must be readable by the
 /*	recipient, and their parent directory needs to have "execute"
@@ -45,11 +51,14 @@
 /*	(complete recipient address), \fB$extension\fR (recipient address
 /*	extension), \fB$domain\fR (recipient domain), \fB$local\fR
 /*	(entire recipient address localpart) and
-/*	\fB$recipient_delimiter.\fR The forms \fI${name?value}\fR and
-/*	\fI${name:value}\fR expand conditionally to \fIvalue\fR when
-/*	\fI$name\fR is (is not) defined.
-/*	Characters that may have special meaning to the shell or file system
-/*	are replaced by underscores.  The list of acceptable characters
+/*	\fB$recipient_delimiter.\fR The forms \fI${name?value}\fR (or
+/*	\fI${name?{value}}\fR (Postfix 3.0 and later)) and \fI${name:value}\fR
+/*	(or \fI${name:{value}}\fR (Postfix 3.0 and later)) expand conditionally
+/*	to \fIvalue\fR when \fI$name\fR is (is not) defined.
+/*	The form \fI${name?{value1}:{value2}}\fR (Postfix 3.0 and later) expands
+/*	conditionally to \fIvalue1\fR when \fI$name\fR is defined, or \fIvalue2\fR
+/*	otherwise. Characters that may have special meaning to the shell or file
+/*	system are replaced with underscores.  The list of acceptable characters
 /*	is specified with the \fBforward_expansion_filter\fR configuration
 /*	parameter.
 /*
@@ -101,18 +110,23 @@
 /*	ending in \fB/\fR for \fBqmail\fR-compatible \fBmaildir\fR delivery.
 /*
 /*	Mailbox delivery can be delegated to an external command specified
-/*	with the \fBmailbox_command\fR configuration parameter. The command
+/*	with the \fBmailbox_command_maps\fR and \fBmailbox_command\fR
+/*	configuration parameters. The command
 /*	executes with the privileges of the recipient user (exceptions:
 /*	secondary groups are not enabled; in case of delivery as root,
 /*	the command executes with the privileges of \fBdefault_privs\fR).
 /*
 /*	Mailbox delivery can be delegated to alternative message transports
 /*	specified in the \fBmaster.cf\fR file.
-/*	The \fBmailbox_transport\fR configuration parameter specifies a
+/*	The \fBmailbox_transport_maps\fR and \fBmailbox_transport\fR
+/*	configuration parameters specify an optional
 /*	message transport that is to be used for all local recipients,
 /*	regardless of whether they are found in the UNIX passwd database.
-/*	The \fBfallback_transport\fR parameter specifies a message transport
-/*	for recipients that are not found in the UNIX passwd database.
+/*	The \fBfallback_transport_maps\fR and
+/*	\fBfallback_transport\fR parameters specify an optional
+/*	message transport
+/*	for recipients that are not found in the aliases(5) or UNIX
+/*	passwd database.
 /*
 /*	In the case of UNIX-style mailbox delivery,
 /*	the \fBlocal\fR(8) daemon prepends a "\fBFrom \fIsender time_stamp\fR"
@@ -153,13 +167,16 @@
 /*	address), \fB$extension\fR (recipient address extension),
 /*	\fB$domain\fR (recipient domain), \fB$local\fR (entire
 /*	recipient address localpart) and \fB$recipient_delimiter.\fR
-/*	The forms \fI${name?value}\fR and \fI${name:value}\fR expand
-/*	conditionally to \fIvalue\fR when \fI$name\fR is (is not)
-/*	defined.  Characters that may have special meaning to the
-/*	shell or file system are replaced by underscores.  The list
-/*	of acceptable characters is specified with the
-/*	\fBexecution_directory_expansion_filter\fR configuration
-/*	parameter.
+/*	The forms \fI${name?value}\fR (or \fI${name?{value}}\fR
+/*	(Postfix 3.0 and later)) and \fI${name:value}\fR (or
+/*	\fI${name:{value}}\fR (Postfix 3.0 and later)) expand conditionally
+/*	to \fIvalue\fR when \fI$name\fR is (is not) defined.
+/*	The form \fI${name?{value1}:{value2}}\fR (Postfix 3.0 and later) expands
+/*	conditionally to \fIvalue1\fR when \fI$name\fR is defined, or \fIvalue2\fR
+/*	otherwise. Characters that may have special meaning to the shell or file
+/*	system are replaced with underscores.  The list of acceptable characters
+/*	is specified with the \fBexecution_directory_expansion_filter\fR
+/*	configuration parameter.
 /*
 /*	The command is executed directly where possible. Assistance by the
 /*	shell (\fB/bin/sh\fR on UNIX systems) is used only when the command
@@ -181,7 +198,7 @@
 /*
 /*	A limited amount of message context is exported via environment
 /*	variables. Characters that may have special meaning to the shell
-/*	are replaced by underscores.  The list of acceptable characters
+/*	are replaced with underscores.  The list of acceptable characters
 /*	is specified with the \fBcommand_expansion_filter\fR configuration
 /*	parameter.
 /* .IP \fBSHELL\fR
@@ -199,6 +216,9 @@
 /* .IP \fBLOCAL\fR
 /*	The entire recipient address localpart (text to the left of the
 /*	rightmost @ character).
+/* .IP \fBORIGINAL_RECIPIENT\fR
+/*	The entire recipient address, before any address rewriting
+/*	or aliasing (Postfix 2.5 and later).
 /* .IP \fBRECIPIENT\fR
 /*	The entire recipient address.
 /* .IP \fBSENDER\fR
@@ -286,10 +306,6 @@
 /*	~\fIname\fR/.\fBforward\fR+\fIfoo\fR or in ~\fIname\fR/.\fBforward\fR,
 /*	to the mailbox owned by the user \fIname\fR, or it is sent back as
 /*	undeliverable.
-/*
-/*	In all cases the \fBlocal\fR(8) daemon prepends an optional
-/*	`\fBDelivered-To:\fR header line with the final recipient
-/*	address.
 /* DELIVERY RIGHTS
 /* .ad
 /* .fi
@@ -304,7 +320,8 @@
 /*	RFC 822 (ARPA Internet Text Messages)
 /*	RFC 3463 (Enhanced status codes)
 /* DIAGNOSTICS
-/*	Problems and transactions are logged to \fBsyslogd\fR(8).
+/*	Problems and transactions are logged to \fBsyslogd\fR(8)
+/*	or \fBpostlogd\fR(8).
 /*	Corrupted message files are marked so that the queue
 /*	manager can move them to the \fBcorrupt\fR queue afterwards.
 /*
@@ -351,13 +368,14 @@
 /* .IP "\fBbiff (yes)\fR"
 /*	Whether or not to use the local biff service.
 /* .IP "\fBexpand_owner_alias (no)\fR"
-/*	When delivering to an alias "aliasname" that has an "owner-aliasname"
-/*	companion alias, set the envelope sender address to the expansion
-/*	of the "owner-aliasname" alias.
+/*	When delivering to an alias "\fIaliasname\fR" that has an
+/*	"owner-\fIaliasname\fR" companion alias, set the envelope sender
+/*	address to the expansion of the "owner-\fIaliasname\fR" alias.
 /* .IP "\fBowner_request_special (yes)\fR"
-/*	Give special treatment to owner-listname and listname-request
-/*	address localparts: don't split such addresses when the
-/*	recipient_delimiter is set to "-".
+/*	Enable special treatment for owner-\fIlistname\fR entries in the
+/*	\fBaliases\fR(5) file, and don't split owner-\fIlistname\fR and
+/*	\fIlistname\fR-request address localparts when the recipient_delimiter
+/*	is set to "-".
 /* .IP "\fBsun_mailtool_compatibility (no)\fR"
 /*	Obsolete SUN mailtool compatibility feature.
 /* .PP
@@ -367,18 +385,37 @@
 /*	address (see prepend_delivered_header) only once, at the start of
 /*	a delivery attempt; do not update the Delivered-To: address while
 /*	expanding aliases or .forward files.
+/* .PP
+/*	Available in Postfix version 2.5.3 and later:
+/* .IP "\fBstrict_mailbox_ownership (yes)\fR"
+/*	Defer delivery when a mailbox file is not owned by its recipient.
+/* .IP "\fBreset_owner_alias (no)\fR"
+/*	Reset the \fBlocal\fR(8) delivery agent's idea of the owner-alias
+/*	attribute, when delivering mail to a child alias that does not have
+/*	its own owner alias.
+/* .PP
+/*	Available in Postfix version 3.0 and later:
+/* .IP "\fBlocal_delivery_status_filter ($default_delivery_status_filter)\fR"
+/*	Optional filter for the \fBlocal\fR(8) delivery agent to change the
+/*	status code or explanatory text of successful or unsuccessful
+/*	deliveries.
 /* DELIVERY METHOD CONTROLS
 /* .ad
 /* .fi
 /*	The precedence of \fBlocal\fR(8) delivery methods from high to low is:
-/*	aliases, .forward files, mailbox_transport, mailbox_command_maps,
-/*	mailbox_command, home_mailbox, mail_spool_directory, fallback_transport
-/*	and luser_relay.
+/*	aliases, .forward files, mailbox_transport_maps,
+/*	mailbox_transport, mailbox_command_maps, mailbox_command,
+/*	home_mailbox, mail_spool_directory, fallback_transport_maps,
+/*	fallback_transport, and luser_relay.
 /* .IP "\fBalias_maps (see 'postconf -d' output)\fR"
 /*	The alias databases that are used for \fBlocal\fR(8) delivery.
 /* .IP "\fBforward_path (see 'postconf -d' output)\fR"
 /*	The \fBlocal\fR(8) delivery agent search list for finding a .forward
 /*	file with user-specified delivery methods.
+/* .IP "\fBmailbox_transport_maps (empty)\fR"
+/*	Optional lookup tables with per-recipient message delivery
+/*	transports to use for \fBlocal\fR(8) mailbox delivery, whether or not the
+/*	recipients are found in the UNIX passwd database.
 /* .IP "\fBmailbox_transport (empty)\fR"
 /*	Optional message delivery transport that the \fBlocal\fR(8) delivery
 /*	agent should use for mailbox delivery to all local recipients,
@@ -394,17 +431,21 @@
 /*	home directory.
 /* .IP "\fBmail_spool_directory (see 'postconf -d' output)\fR"
 /*	The directory where \fBlocal\fR(8) UNIX-style mailboxes are kept.
+/* .IP "\fBfallback_transport_maps (empty)\fR"
+/*	Optional lookup tables with per-recipient message delivery
+/*	transports for recipients that the \fBlocal\fR(8) delivery agent could
+/*	not find in the \fBaliases\fR(5) or UNIX password database.
 /* .IP "\fBfallback_transport (empty)\fR"
 /*	Optional message delivery transport that the \fBlocal\fR(8) delivery
 /*	agent should use for names that are not found in the \fBaliases\fR(5)
-/*	database or in the UNIX passwd database.
+/*	or UNIX password database.
 /* .IP "\fBluser_relay (empty)\fR"
 /*	Optional catch-all destination for unknown \fBlocal\fR(8) recipients.
 /* .PP
 /*	Available in Postfix version 2.2 and later:
 /* .IP "\fBcommand_execution_directory (empty)\fR"
 /*	The \fBlocal\fR(8) delivery agent working directory for delivery to
-/*	external command.
+/*	external commands.
 /* MAILBOX LOCKING CONTROLS
 /* .ad
 /* .fi
@@ -427,6 +468,11 @@
 /*	The maximal number of addresses remembered by the address
 /*	duplicate filter for \fBaliases\fR(5) or \fBvirtual\fR(5) alias expansion, or
 /*	for \fBshowq\fR(8) queue displays.
+/* .IP "\fBmailbox_size_limit (51200000)\fR"
+/*	The maximal size of any \fBlocal\fR(8) individual mailbox or maildir
+/*	file, or zero (no limit).
+/* .PP
+/*	Implemented in the qmgr(8) daemon:
 /* .IP "\fBlocal_destination_concurrency_limit (2)\fR"
 /*	The maximal number of parallel deliveries via the local mail
 /*	delivery transport to the same recipient (when
@@ -436,9 +482,6 @@
 /* .IP "\fBlocal_destination_recipient_limit (1)\fR"
 /*	The maximal number of recipients per message delivery via the
 /*	local mail delivery transport.
-/* .IP "\fBmailbox_size_limit (51200000)\fR"
-/*	The maximal size of any \fBlocal\fR(8) individual mailbox or maildir
-/*	file, or zero (no limit).
 /* SECURITY CONTROLS
 /* .ad
 /* .fi
@@ -448,10 +491,10 @@
 /*	Restrict \fBlocal\fR(8) mail delivery to external files.
 /* .IP "\fBcommand_expansion_filter (see 'postconf -d' output)\fR"
 /*	Restrict the characters that the \fBlocal\fR(8) delivery agent allows in
-/*	$name expansions of $mailbox_command.
+/*	$name expansions of $mailbox_command and $command_execution_directory.
 /* .IP "\fBdefault_privs (nobody)\fR"
 /*	The default rights used by the \fBlocal\fR(8) delivery agent for delivery
-/*	to external file or command.
+/*	to an external file or command.
 /* .IP "\fBforward_expansion_filter (see 'postconf -d' output)\fR"
 /*	Restrict the characters that the \fBlocal\fR(8) delivery agent allows in
 /*	$name expansions of $forward_path.
@@ -460,6 +503,10 @@
 /* .IP "\fBexecution_directory_expansion_filter (see 'postconf -d' output)\fR"
 /*	Restrict the characters that the \fBlocal\fR(8) delivery agent allows
 /*	in $name expansions of $command_execution_directory.
+/* .PP
+/*	Available in Postfix version 2.5.3 and later:
+/* .IP "\fBstrict_mailbox_ownership (yes)\fR"
+/*	Defer delivery when a mailbox file is not owned by its recipient.
 /* MISCELLANEOUS CONTROLS
 /* .ad
 /* .fi
@@ -469,6 +516,9 @@
 /* .IP "\fBdaemon_timeout (18000s)\fR"
 /*	How much time a Postfix daemon process may take to handle a
 /*	request before it is terminated by a built-in watchdog timer.
+/* .IP "\fBdelay_logging_resolution_limit (2)\fR"
+/*	The maximal number of digits after the decimal point when logging
+/*	sub-second delay values.
 /* .IP "\fBexport_environment (see 'postconf -d' output)\fR"
 /*	The list of environment variables that a Postfix process will export
 /*	to non-Postfix processes.
@@ -476,13 +526,13 @@
 /*	The time limit for sending or receiving information over an internal
 /*	communication channel.
 /* .IP "\fBlocal_command_shell (empty)\fR"
-/*	Optional shell program for \fBlocal\fR(8) delivery to non-Postfix command.
+/*	Optional shell program for \fBlocal\fR(8) delivery to non-Postfix commands.
 /* .IP "\fBmax_idle (100s)\fR"
-/*	The maximum amount of time that an idle Postfix daemon process
-/*	waits for the next service request before exiting.
+/*	The maximum amount of time that an idle Postfix daemon process waits
+/*	for an incoming connection before terminating voluntarily.
 /* .IP "\fBmax_use (100)\fR"
-/*	The maximal number of connection requests before a Postfix daemon
-/*	process terminates.
+/*	The maximal number of incoming connections that a Postfix daemon
+/*	process will service before terminating voluntarily.
 /* .IP "\fBprepend_delivered_header (command, file, forward)\fR"
 /*	The message delivery contexts where the Postfix \fBlocal\fR(8) delivery
 /*	agent prepends a Delivered-To:  message header with the address
@@ -497,15 +547,29 @@
 /* .IP "\fBqueue_directory (see 'postconf -d' output)\fR"
 /*	The location of the Postfix top-level queue directory.
 /* .IP "\fBrecipient_delimiter (empty)\fR"
-/*	The separator between user names and address extensions (user+foo).
+/*	The set of characters that can separate an email address
+/*	localpart, user name, or a .forward file name from its extension.
 /* .IP "\fBrequire_home_directory (no)\fR"
-/*	Whether or not a \fBlocal\fR(8) recipient's home directory must exist
+/*	Require that a \fBlocal\fR(8) recipient's home directory exists
 /*	before mail delivery is attempted.
 /* .IP "\fBsyslog_facility (mail)\fR"
 /*	The syslog facility of Postfix logging.
-/* .IP "\fBsyslog_name (postfix)\fR"
-/*	The mail system name that is prepended to the process name in syslog
-/*	records, so that "smtpd" becomes, for example, "postfix/smtpd".
+/* .IP "\fBsyslog_name (see 'postconf -d' output)\fR"
+/*	A prefix that is prepended to the process name in syslog
+/*	records, so that, for example, "smtpd" becomes "prefix/smtpd".
+/* .PP
+/*	Available in Postfix version 3.3 and later:
+/* .IP "\fBenable_original_recipient (yes)\fR"
+/*	Enable support for the original recipient address after an
+/*	address is rewritten to a different address (for example with
+/*	aliasing or with canonical mapping).
+/* .IP "\fBservice_name (read-only)\fR"
+/*	The master.cf service name of a Postfix daemon process.
+/* .PP
+/*	Available in Postfix 3.5 and later:
+/* .IP "\fBinfo_log_address_format (external)\fR"
+/*	The email address form that will be used in non-debug logging
+/*	(info, warning, etc.).
 /* FILES
 /*	The following are examples; details differ between systems.
 /*	$HOME/.forward, per-user aliasing
@@ -519,6 +583,7 @@
 /*	aliases(5), format of alias database
 /*	postconf(5), configuration parameters
 /*	master(5), generic daemon options
+/*	postlogd(8), Postfix logging
 /*	syslogd(8), system logging
 /* LICENSE
 /* .ad
@@ -537,6 +602,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System library. */
@@ -572,6 +642,7 @@
 #include <mail_conf.h>
 #include <been_here.h>
 #include <mail_params.h>
+#include <mail_version.h>
 #include <ext_prop.h>
 #include <maps.h>
 #include <flush_clnt.h>
@@ -591,7 +662,7 @@ char   *var_allow_commands;
 char   *var_allow_files;
 char   *var_alias_maps;
 int     var_dup_filter_limit;
-int     var_command_maxtime;
+int     var_command_maxtime;		/* You can now leave this here. */
 char   *var_home_mailbox;
 char   *var_mailbox_command;
 char   *var_mailbox_cmd_maps;
@@ -601,7 +672,9 @@ char   *var_luser_relay;
 int     var_biff;
 char   *var_mail_spool_dir;
 char   *var_mailbox_transport;
+char   *var_mbox_transp_maps;
 char   *var_fallback_transport;
+char   *var_fbck_transp_maps;
 char   *var_exec_directory;
 char   *var_exec_exp_filter;
 char   *var_forward_path;
@@ -613,8 +686,10 @@ char   *var_deliver_hdr;
 int     var_stat_home_dir;
 int     var_mailtool_compat;
 char   *var_mailbox_lock;
-int     var_mailbox_limit;
+long    var_mailbox_limit;
 bool    var_frozen_delivered;
+bool    var_reset_owner_attr;
+bool    var_strict_mbox_owner;
 
 int     local_cmd_deliver_mask;
 int     local_file_deliver_mask;
@@ -622,12 +697,13 @@ int     local_ext_prop_mask;
 int     local_deliver_hdr_mask;
 int     local_mbox_lock_mask;
 MAPS   *alias_maps;
+char   *var_local_dsn_filter;
 
 /* local_deliver - deliver message with extreme prejudice */
 
 static int local_deliver(DELIVER_REQUEST *rqst, char *service)
 {
-    char   *myname = "local_deliver";
+    const char *myname = "local_deliver";
     RECIPIENT *rcpt_end = rqst->rcpt_list.info + rqst->rcpt_list.len;
     RECIPIENT *rcpt;
     int     rcpt_stat;
@@ -659,15 +735,17 @@ static int local_deliver(DELIVER_REQUEST *rqst, char *service)
     state.msg_attr.fp = rqst->fp;
     state.msg_attr.offset = rqst->data_offset;
     state.msg_attr.encoding = rqst->encoding;
+    state.msg_attr.smtputf8 = rqst->smtputf8;
     state.msg_attr.sender = rqst->sender;
     state.msg_attr.dsn_envid = rqst->dsn_envid;
     state.msg_attr.dsn_ret = rqst->dsn_ret;
     state.msg_attr.relay = service;
-    state.msg_attr.arrival_time = rqst->arrival_time;
+    state.msg_attr.msg_stats = rqst->msg_stats;
     state.msg_attr.request = rqst;
     RESET_OWNER_ATTR(state.msg_attr, state.level);
     RESET_USER_ATTR(usr_attr, state.level);
-    state.loop_info = delivered_init(state.msg_attr);	/* delivered-to */
+    state.loop_info = delivered_hdr_init(rqst->fp, rqst->data_offset,
+					 FOLD_ADDR_ALL);
     state.request = rqst;
 
     /*
@@ -691,7 +769,7 @@ static int local_deliver(DELIVER_REQUEST *rqst, char *service)
     /*
      * Clean up.
      */
-    delivered_free(state.loop_info);
+    delivered_hdr_free(state.loop_info);
     deliver_attr_free(&state.msg_attr);
 
     return (msg_stat);
@@ -727,19 +805,19 @@ static void local_service(VSTREAM *stream, char *service, char **argv)
 
 static void local_mask_init(void)
 {
-    static NAME_MASK file_mask[] = {
+    static const NAME_MASK file_mask[] = {
 	"alias", EXPAND_TYPE_ALIAS,
 	"forward", EXPAND_TYPE_FWD,
 	"include", EXPAND_TYPE_INCL,
 	0,
     };
-    static NAME_MASK command_mask[] = {
+    static const NAME_MASK command_mask[] = {
 	"alias", EXPAND_TYPE_ALIAS,
 	"forward", EXPAND_TYPE_FWD,
 	"include", EXPAND_TYPE_INCL,
 	0,
     };
-    static NAME_MASK deliver_mask[] = {
+    static const NAME_MASK deliver_mask[] = {
 	"command", DELIVER_HDR_CMD,
 	"file", DELIVER_HDR_FILE,
 	"forward", DELIVER_HDR_FWD,
@@ -807,32 +885,42 @@ static void pre_init(char *unused_name, char **unused_argv)
      * because that prohibits the delivery agent from updating the queue
      * file.
      */
-    if (var_mailbox_limit) {
-	if (var_mailbox_limit < var_message_limit || var_message_limit == 0)
-	    msg_fatal("main.cf configuration error: %s is smaller than %s",
+    if (ENFORCING_SIZE_LIMIT(var_mailbox_limit)) {
+	if (!ENFORCING_SIZE_LIMIT(var_message_limit))
+	    msg_fatal("configuration error: %s is limited but %s is "
+		      "unlimited", VAR_MAILBOX_LIMIT, VAR_MESSAGE_LIMIT);
+	if (var_mailbox_limit < var_message_limit)
+	    msg_fatal("configuration error: %s is smaller than %s",
 		      VAR_MAILBOX_LIMIT, VAR_MESSAGE_LIMIT);
 	set_file_limit(var_mailbox_limit);
     }
     alias_maps = maps_create("aliases", var_alias_maps,
-			     DICT_FLAG_LOCK | DICT_FLAG_PARANOID);
+			     DICT_FLAG_LOCK | DICT_FLAG_PARANOID
+			     | DICT_FLAG_FOLD_FIX
+			     | DICT_FLAG_UTF8_REQUEST);
 
     flush_init();
 }
 
+MAIL_VERSION_STAMP_DECLARE;
+
 /* main - pass control to the single-threaded skeleton */
 
 int     main(int argc, char **argv)
 {
-    static CONFIG_TIME_TABLE time_table[] = {
+    static const CONFIG_TIME_TABLE time_table[] = {
 	VAR_COMMAND_MAXTIME, DEF_COMMAND_MAXTIME, &var_command_maxtime, 1, 0,
 	0,
     };
-    static CONFIG_INT_TABLE int_table[] = {
+    static const CONFIG_INT_TABLE int_table[] = {
 	VAR_DUP_FILTER_LIMIT, DEF_DUP_FILTER_LIMIT, &var_dup_filter_limit, 0, 0,
+	0,
+    };
+    static const CONFIG_LONG_TABLE long_table[] = {
 	VAR_MAILBOX_LIMIT, DEF_MAILBOX_LIMIT, &var_mailbox_limit, 0, 0,
 	0,
     };
-    static CONFIG_STR_TABLE str_table[] = {
+    static const CONFIG_STR_TABLE str_table[] = {
 	VAR_ALIAS_MAPS, DEF_ALIAS_MAPS, &var_alias_maps, 0, 0,
 	VAR_HOME_MAILBOX, DEF_HOME_MAILBOX, &var_home_mailbox, 0, 0,
 	VAR_ALLOW_COMMANDS, DEF_ALLOW_COMMANDS, &var_allow_commands, 0, 0,
@@ -840,43 +928,56 @@ int     main(int argc, char **argv)
 	VAR_LOCAL_CMD_SHELL, DEF_LOCAL_CMD_SHELL, &var_local_cmd_shell, 0, 0,
 	VAR_MAIL_SPOOL_DIR, DEF_MAIL_SPOOL_DIR, &var_mail_spool_dir, 0, 0,
 	VAR_MAILBOX_TRANSP, DEF_MAILBOX_TRANSP, &var_mailbox_transport, 0, 0,
+	VAR_MBOX_TRANSP_MAPS, DEF_MBOX_TRANSP_MAPS, &var_mbox_transp_maps, 0, 0,
 	VAR_FALLBACK_TRANSP, DEF_FALLBACK_TRANSP, &var_fallback_transport, 0, 0,
+	VAR_FBCK_TRANSP_MAPS, DEF_FBCK_TRANSP_MAPS, &var_fbck_transp_maps, 0, 0,
 	VAR_CMD_EXP_FILTER, DEF_CMD_EXP_FILTER, &var_cmd_exp_filter, 1, 0,
 	VAR_FWD_EXP_FILTER, DEF_FWD_EXP_FILTER, &var_fwd_exp_filter, 1, 0,
 	VAR_EXEC_EXP_FILTER, DEF_EXEC_EXP_FILTER, &var_exec_exp_filter, 1, 0,
 	VAR_PROP_EXTENSION, DEF_PROP_EXTENSION, &var_prop_extension, 0, 0,
 	VAR_DELIVER_HDR, DEF_DELIVER_HDR, &var_deliver_hdr, 0, 0,
 	VAR_MAILBOX_LOCK, DEF_MAILBOX_LOCK, &var_mailbox_lock, 1, 0,
+	VAR_MAILBOX_CMD_MAPS, DEF_MAILBOX_CMD_MAPS, &var_mailbox_cmd_maps, 0, 0,
+	VAR_LOCAL_DSN_FILTER, DEF_LOCAL_DSN_FILTER, &var_local_dsn_filter, 0, 0,
 	0,
     };
-    static CONFIG_BOOL_TABLE bool_table[] = {
+    static const CONFIG_BOOL_TABLE bool_table[] = {
 	VAR_BIFF, DEF_BIFF, &var_biff,
 	VAR_EXP_OWN_ALIAS, DEF_EXP_OWN_ALIAS, &var_exp_own_alias,
 	VAR_STAT_HOME_DIR, DEF_STAT_HOME_DIR, &var_stat_home_dir,
 	VAR_MAILTOOL_COMPAT, DEF_MAILTOOL_COMPAT, &var_mailtool_compat,
 	VAR_FROZEN_DELIVERED, DEF_FROZEN_DELIVERED, &var_frozen_delivered,
+	VAR_RESET_OWNER_ATTR, DEF_RESET_OWNER_ATTR, &var_reset_owner_attr,
+	VAR_STRICT_MBOX_OWNER, DEF_STRICT_MBOX_OWNER, &var_strict_mbox_owner,
 	0,
     };
 
     /* Suppress $name expansion upon loading. */
-    static CONFIG_RAW_TABLE raw_table[] = {
+    static const CONFIG_RAW_TABLE raw_table[] = {
 	VAR_EXEC_DIRECTORY, DEF_EXEC_DIRECTORY, &var_exec_directory, 0, 0,
 	VAR_FORWARD_PATH, DEF_FORWARD_PATH, &var_forward_path, 0, 0,
 	VAR_MAILBOX_COMMAND, DEF_MAILBOX_COMMAND, &var_mailbox_command, 0, 0,
-	VAR_MAILBOX_CMD_MAPS, DEF_MAILBOX_CMD_MAPS, &var_mailbox_cmd_maps, 0, 0,
 	VAR_LUSER_RELAY, DEF_LUSER_RELAY, &var_luser_relay, 0, 0,
 	0,
     };
 
+    /*
+     * Fingerprint executables and core dumps.
+     */
+    MAIL_VERSION_STAMP_ALLOCATE;
+
     single_server_main(argc, argv, local_service,
-		       MAIL_SERVER_INT_TABLE, int_table,
-		       MAIL_SERVER_STR_TABLE, str_table,
-		       MAIL_SERVER_RAW_TABLE, raw_table,
-		       MAIL_SERVER_BOOL_TABLE, bool_table,
-		       MAIL_SERVER_TIME_TABLE, time_table,
-		       MAIL_SERVER_PRE_INIT, pre_init,
-		       MAIL_SERVER_POST_INIT, post_init,
-		       MAIL_SERVER_PRE_ACCEPT, pre_accept,
-		       MAIL_SERVER_PRIVILEGED,
+		       CA_MAIL_SERVER_INT_TABLE(int_table),
+		       CA_MAIL_SERVER_LONG_TABLE(long_table),
+		       CA_MAIL_SERVER_STR_TABLE(str_table),
+		       CA_MAIL_SERVER_RAW_TABLE(raw_table),
+		       CA_MAIL_SERVER_BOOL_TABLE(bool_table),
+		       CA_MAIL_SERVER_TIME_TABLE(time_table),
+		       CA_MAIL_SERVER_PRE_INIT(pre_init),
+		       CA_MAIL_SERVER_POST_INIT(post_init),
+		       CA_MAIL_SERVER_PRE_ACCEPT(pre_accept),
+		       CA_MAIL_SERVER_PRIVILEGED,
+		       CA_MAIL_SERVER_BOUNCE_INIT(VAR_LOCAL_DSN_FILTER,
+						  &var_local_dsn_filter),
 		       0);
 }
